@@ -360,7 +360,7 @@ dex_future_discard (DexFuture *future,
 }
 
 /**
- * dex_future_then:
+ * dex_future_then: (constructor)
  * @future: (transfer full): a #DexFuture
  * @callback: (scope async): a callback to execute
  * @callback_data: closure data for @callback
@@ -390,7 +390,7 @@ DexFuture *
 }
 
 /**
- * dex_future_then_loop:
+ * dex_future_then_loop: (constructor)
  * @future: (transfer full): a #DexFuture
  * @callback: (scope async): a callback to execute
  * @callback_data: closure data for @callback
@@ -422,7 +422,7 @@ DexFuture *
 }
 
 /**
- * dex_future_catch_loop:
+ * dex_future_catch_loop: (constructor)
  * @future: (transfer full): a #DexFuture
  * @callback: (scope async): a callback to execute
  * @callback_data: closure data for @callback
@@ -454,7 +454,7 @@ DexFuture *
 }
 
 /**
- * dex_future_finally_loop:
+ * dex_future_finally_loop: (constructor)
  * @future: (transfer full): a #DexFuture
  * @callback: (scope async): a callback to execute
  * @callback_data: closure data for @callback
@@ -486,7 +486,7 @@ DexFuture *
 }
 
 /**
- * dex_future_catch:
+ * dex_future_catch: (constructor)
  * @future: (transfer full): a #DexFuture
  * @callback: (scope async): a callback to execute
  * @callback_data: closure data for @callback
@@ -516,7 +516,7 @@ DexFuture *
 }
 
 /**
- * dex_future_finally:
+ * dex_future_finally: (constructor)
  * @future: (transfer full): a #DexFuture
  * @callback: (scope async): a callback to execute
  * @callback_data: closure data for @callback
@@ -544,7 +544,7 @@ DexFuture *
 }
 
 /**
- * dex_future_all: (skip)
+ * dex_future_all: (constructor)
  * @first_future: (transfer full): a #DexFuture
  * @...: a %NULL terminated list of futures
  *
@@ -555,7 +555,7 @@ DexFuture *
  * the exact values of the dependent futures. The value of the future
  * if resolved will be a %G_TYPE_BOOLEAN of %TRUE.
  *
- * Returns: (transfer full) (type DexFutureSet): a #DexFuture
+ * Returns: (transfer full) (type DexFuture): a #DexFutureSet
  */
 DexFuture *
 (dex_future_all) (DexFuture *first_future,
@@ -572,7 +572,7 @@ DexFuture *
 }
 
 /**
- * dex_future_any: (skip)
+ * dex_future_any: (constructor)
  * @first_future: (transfer full): a #DexFuture
  * @...: a %NULL terminated list of futures
  *
@@ -581,7 +581,7 @@ DexFuture *
  *
  * If no futures resolve, then the future will reject.
  *
- * Returns: (transfer full) (type DexFutureSet): a #DexFuture
+ * Returns: (transfer full) (type DexFuture): a #DexFutureSet
  */
 DexFuture *
 (dex_future_any) (DexFuture *first_future,
@@ -600,7 +600,7 @@ DexFuture *
 }
 
 /**
- * dex_future_all_race:
+ * dex_future_all_race: (constructor)
  * @first_future: (transfer full): a #DexFuture
  * @...: a %NULL terminated list of futures
  *
@@ -614,7 +614,7 @@ DexFuture *
  * Since the futures race to complete, some futures retrieved with the
  * dex_future_set_get_future() API will still be %DEX_FUTURE_STATUS_PENDING.
  *
- * Returns: (transfer full) (type DexFutureSet): a #DexFuture
+ * Returns: (transfer full) (type DexFuture): a #DexFutureSet
  */
 DexFuture *
 (dex_future_all_race) (DexFuture *first_future,
@@ -633,14 +633,14 @@ DexFuture *
 }
 
 /**
- * dex_future_first: (skip)
+ * dex_future_first: (constructor)
  * @first_future: (transfer full): a #DexFuture
  * @...: a %NULL terminated list of futures
  *
  * Creates a new #DexFuture that resolves or rejects as soon as the
  * first dependent future resolves or rejects, sharing the same result.
  *
- * Returns: (transfer full) (type DexFutureSet): a #DexFuture
+ * Returns: (transfer full) (type DexFuture): a #DexFutureSet
  */
 DexFuture *
 (dex_future_first) (DexFuture *first_future,
@@ -660,7 +660,7 @@ DexFuture *
 }
 
 /**
- * dex_future_firstv: (rename-to dex_future_first)
+ * dex_future_firstv: (rename-to dex_future_first) (constructor)
  * @futures: (array length=n_futures) (transfer none): an array of futures
  * @n_futures: the number of futures
  *
@@ -680,7 +680,7 @@ DexFuture *
 }
 
 /**
- * dex_future_anyv: (rename-to dex_future_any)
+ * dex_future_anyv: (rename-to dex_future_any) (constructor)
  * @futures: (array length=n_futures) (transfer none): an array of futures
  * @n_futures: the number of futures
  *
@@ -700,7 +700,7 @@ DexFuture *
 }
 
 /**
- * dex_future_all_racev: (rename-to dex_future_all_race)
+ * dex_future_all_racev: (rename-to dex_future_all_race) (constructor)
  * @futures: (array length=n_futures) (transfer none): an array of futures
  * @n_futures: the number of futures
  *
@@ -720,7 +720,7 @@ DexFuture *
 }
 
 /**
- * dex_future_allv: (rename-to dex_future_all)
+ * dex_future_allv: (rename-to dex_future_all) (constructor)
  * @futures: (array length=n_futures) (transfer none): an array of futures
  * @n_futures: the number of futures
  *
@@ -790,7 +790,7 @@ DexFuture *
 }
 
 /**
- * dex_future_new_for_error:
+ * dex_future_new_for_error: (constructor)
  * @error: (transfer full): a #GError
  *
  * Creates a read-only #DexFuture that has rejected.
@@ -1036,6 +1036,28 @@ DexFuture *
   return ret;
 }
 
+/**
+ * dex_future_new_take_variant: (constructor) (skip)
+ * @v_variant: the variant to take ownership of
+ *
+ * Creates a new #DexFuture that is resolved with @v_variant.
+ *
+ * Returns: (transfer full): a resolved #DexFuture
+ */
+DexFuture *
+(dex_future_new_take_variant) (GVariant *v_variant)
+{
+  GValue gvalue = G_VALUE_INIT;
+  DexFuture *ret;
+
+  g_value_init (&gvalue, G_TYPE_VARIANT);
+  g_value_take_variant (&gvalue, v_variant);
+  ret = dex_future_new_for_value (&gvalue);
+  g_value_unset (&gvalue);
+
+  return ret;
+}
+
 /**
  * dex_future_new_for_pointer: (constructor)
  * @pointer: the resolved future value as a pointer
@@ -1134,6 +1156,48 @@ DexFuture *
   return dex_future_new_for_error (error);
 }
 
+/**
+ * dex_future_new_for_errno:
+ * @errno_: the `errno` to use for rejection
+ *
+ * Creates a new rejected future using @errno_ as the value
+ * of errno for the GError.
+ *
+ * The resulting error domain will be %G_IO_ERROR.
+ *
+ * Returns: (transfer full): a rejected #DexFuture.
+ *
+ * Since: 0.4
+ */
+DexFuture *
+(dex_future_new_for_errno) (int errno_)
+{
+  /* NOTE: We might be able to cache some common rejections
+   * by errno to avoid re-creating them.
+   */
+
+  return dex_future_new_for_error (g_error_new_literal (G_IO_ERROR,
+                                                        g_io_error_from_errno (errno_),
+                                                        g_strerror (errno_)));
+}
+
+/**
+ * dex_future_new_infinite:
+ *
+ * Creates an infinite future that will never resolve or reject. This can
+ * be useful when you want to mock a situation of "run forever" unless
+ * another future rejects or resolves.
+ *
+ * Returns: (transfer full): a #DexFuture that will never complete or reject
+ *
+ * Since: 0.4
+ */
+DexFuture *
+dex_future_new_infinite (void)
+{
+  return (DexFuture *)dex_object_create_instance (DEX_TYPE_FUTURE);
+}
+
 static const GValue *
 dex_await_check (DexFuture  *future,
                  GType       type,
@@ -1161,7 +1225,7 @@ dex_await_check (DexFuture  *future,
 }
 
 /**
- * dex_await_pointer: (method) (skip)
+ * dex_await_pointer: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1188,7 +1252,7 @@ dex_await_pointer (DexFuture  *future,
 }
 
 /**
- * dex_await_int: (method) (skip)
+ * dex_await_int: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1216,7 +1280,7 @@ dex_await_int (DexFuture  *future,
 }
 
 /**
- * dex_await_uint: (method) (skip)
+ * dex_await_uint: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1244,7 +1308,7 @@ dex_await_uint (DexFuture  *future,
 }
 
 /**
- * dex_await_int64: (method) (skip)
+ * dex_await_int64: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1272,7 +1336,7 @@ dex_await_int64 (DexFuture  *future,
 }
 
 /**
- * dex_await_uint64: (method) (skip)
+ * dex_await_uint64: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1300,7 +1364,7 @@ dex_await_uint64 (DexFuture  *future,
 }
 
 /**
- * dex_await_double: (method) (skip)
+ * dex_await_double: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1328,7 +1392,7 @@ dex_await_double (DexFuture  *future,
 }
 
 /**
- * dex_await_float: (method) (skip)
+ * dex_await_float: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1356,7 +1420,7 @@ dex_await_float (DexFuture  *future,
 }
 
 /**
- * dex_await_boxed: (method) (skip)
+ * dex_await_boxed: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1382,7 +1446,35 @@ dex_await_boxed (DexFuture  *future,
 }
 
 /**
- * dex_await_object: (method) (skip)
+ * dex_await_variant: (method)
+ * @future: (transfer full): a #DexFuture
+ * @error: a location for a #GError
+ *
+ * Awaits on @future and returns the %G_TYPE_VARIANT based result.
+ *
+ * Returns: (transfer full): the variant result, or %NULL and @error is set.
+ *
+ * Since: 0.4
+ */
+GVariant *
+dex_await_variant (DexFuture  *future,
+                   GError    **error)
+{
+  const GValue *value;
+  GVariant *ret = NULL;
+
+  g_return_val_if_fail (DEX_IS_FUTURE (future), 0);
+
+  if ((value = dex_await_check (future, G_TYPE_VARIANT, error)))
+    ret = g_value_dup_variant (value);
+
+  dex_unref (future);
+
+  return ret;
+}
+
+/**
+ * dex_await_object: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1408,7 +1500,7 @@ dex_await_object (DexFuture  *future,
 }
 
 /**
- * dex_await_boolean: (method) (skip)
+ * dex_await_boolean: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1436,7 +1528,7 @@ dex_await_boolean (DexFuture  *future,
 }
 
 /**
- * dex_await_string: (method) (skip)
+ * dex_await_string: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1464,7 +1556,7 @@ dex_await_string (DexFuture  *future,
 }
 
 /**
- * dex_await_enum: (method) (skip)
+ * dex_await_enum: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
@@ -1492,7 +1584,7 @@ dex_await_enum (DexFuture  *future,
 }
 
 /**
- * dex_await_flags: (method) (skip)
+ * dex_await_flags: (method)
  * @future: (transfer full): a #DexFuture
  * @error: a location for a #GError
  *
