@@ -50,6 +50,8 @@ typedef struct _DexFuture DexFuture;
  *
  * This is the expected way to handle completion of a future when not using
  * #DexFiber via dex_scheduler_spawn().
+ *
+ * Returns: (transfer full) (nullable): a #DexFuture or %NULL
  */
 typedef DexFuture *(*DexFutureCallback) (DexFuture *future,
                                          gpointer   user_data);
@@ -66,11 +68,15 @@ DexFuture       *dex_future_new_for_value   (const GValue       *value);
 DEX_AVAILABLE_IN_ALL
 DexFuture       *dex_future_new_for_error   (GError             *error);
 DEX_AVAILABLE_IN_ALL
+DexFuture       *dex_future_new_infinite    (void);
+DEX_AVAILABLE_IN_ALL
 DexFuture       *dex_future_new_reject      (GQuark              domain,
                                              int                 error_code,
                                              const char         *format,
                                              ...) G_GNUC_PRINTF (3, 4);
 DEX_AVAILABLE_IN_ALL
+DexFuture       *dex_future_new_for_errno   (int                 errno_);
+DEX_AVAILABLE_IN_ALL
 DexFuture       *dex_future_new_for_string  (const char         *string);
 DEX_AVAILABLE_IN_ALL
 DexFuture       *dex_future_new_take_string (char               *string);
@@ -89,6 +95,8 @@ DexFuture       *dex_future_new_for_double  (double              v_double);
 DEX_AVAILABLE_IN_ALL
 DexFuture       *dex_future_new_for_float   (float               v_float);
 DEX_AVAILABLE_IN_ALL
+DexFuture       *dex_future_new_take_variant(GVariant           *v_variant);
+DEX_AVAILABLE_IN_ALL
 DexFuture       *dex_future_new_take_boxed  (GType               boxed_type,
                                              gpointer            value);
 DEX_AVAILABLE_IN_ALL
@@ -193,6 +201,9 @@ DEX_AVAILABLE_IN_ALL
 guint64          dex_await_uint64           (DexFuture          *future,
                                              GError            **error);
 DEX_AVAILABLE_IN_ALL
+GVariant        *dex_await_variant          (DexFuture          *future,
+                                             GError            **error);
+DEX_AVAILABLE_IN_ALL
 gpointer         dex_await_boxed            (DexFuture          *future,
                                              GError            **error);
 DEX_AVAILABLE_IN_ALL
@@ -221,6 +232,7 @@ gpointer         dex_await_object           (DexFuture          *future,
 # define dex_future_firstv(...) _DEX_FUTURE_NEW(firstv, __VA_ARGS__)
 # define dex_future_new_for_value(...) _DEX_FUTURE_NEW(new_for_value, __VA_ARGS__)
 # define dex_future_new_for_error(...) _DEX_FUTURE_NEW(new_for_error, __VA_ARGS__)
+# define dex_future_new_for_errno(...) _DEX_FUTURE_NEW(new_for_errno, __VA_ARGS__)
 # define dex_future_new_for_string(...) _DEX_FUTURE_NEW(new_for_string, __VA_ARGS__)
 # define dex_future_new_take_string(...) _DEX_FUTURE_NEW(new_take_string, __VA_ARGS__)
 # define dex_future_new_for_int(...) _DEX_FUTURE_NEW(new_for_int, __VA_ARGS__)
@@ -230,6 +242,7 @@ gpointer         dex_await_object           (DexFuture          *future,
 # define dex_future_new_for_boolean(...) _DEX_FUTURE_NEW(new_for_boolean, __VA_ARGS__)
 # define dex_future_new_for_double(...) _DEX_FUTURE_NEW(new_for_double, __VA_ARGS__)
 # define dex_future_new_for_float(...) _DEX_FUTURE_NEW(new_for_float, __VA_ARGS__)
+# define dex_future_new_take_variant(...) _DEX_FUTURE_NEW(new_take_variant, __VA_ARGS__)
 # define dex_future_new_take_boxed(...) _DEX_FUTURE_NEW(new_take_boxed, __VA_ARGS__)
 # define dex_future_new_for_object(...) _DEX_FUTURE_NEW(new_for_object, __VA_ARGS__)
 # define dex_future_new_take_object(...) _DEX_FUTURE_NEW(new_take_object, __VA_ARGS__)
