@@ -23,7 +23,9 @@
 
 #include "dex-async-pair-private.h"
 #include "dex-future-private.h"
+#include "dex-future-set.h"
 #include "dex-gio.h"
+#include "dex-promise.h"
 
 typedef struct _DexFileInfoList DexFileInfoList;
 
@@ -920,3 +922,331 @@ dex_file_load_contents_bytes (GFile *file)
 
   return DEX_FUTURE (async_pair);
 }
+
+static void
+dex_dbus_connection_send_message_with_reply_cb (GObject      *object,
+                                                GAsyncResult *result,
+                                                gpointer      user_data)
+{
+  DexAsyncPair *async_pair = user_data;
+  GDBusMessage *message = NULL;
+  GError *error = NULL;
+
+  message = g_dbus_connection_send_message_with_reply_finish (G_DBUS_CONNECTION (object), result, &error);
+
+  if (error == NULL)
+    dex_async_pair_return_object (async_pair, message);
+  else
+    dex_async_pair_return_error (async_pair, error);
+
+  dex_unref (async_pair);
+}
+
+/**
+ * dex_dbus_connection_send_message_with_reply:
+ * @connection: a #GDBusConnection
+ * @message: a #GDBusMessage
+ * @flags:  flags for @message
+ * @timeout_msec: timeout in milliseconds, or -1 for default, or %G_MAXINT
+ *   for no timeout.
+ * @out_serial: (out) (nullable): a location for the message serial number
+ *
+ * Wrapper for g_dbus_connection_send_message_with_reply().
+ *
+ * Returns: (transfer full): a #DexFuture that will resolve to a #GDBusMessage
+ *   or reject with failure.
+ *
+ * Since: 0.4
+ */
+DexFuture *
+dex_dbus_connection_send_message_with_reply (GDBusConnection       *connection,
+                                             GDBusMessage          *message,
+                                             GDBusSendMessageFlags  flags,
+                                             int                    timeout_msec,
+                                             guint32               *out_serial)
+{
+  DexAsyncPair *async_pair;
+
+  g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), NULL);
+  g_return_val_if_fail (G_IS_DBUS_MESSAGE (message), NULL);
+
+  async_pair = (DexAsyncPair *)dex_object_create_instance (DEX_TYPE_ASYNC_PAIR);
+
+  g_dbus_connection_send_message_with_reply (connection,
+                                             message,
+                                             flags,
+                                             timeout_msec,
+                                             out_serial,
+                                             async_pair->cancellable,
+                                             dex_dbus_connection_send_message_with_reply_cb,
+                                             dex_ref (async_pair));
+
+  return DEX_FUTURE (async_pair);
+}
+
+static void
+dex_dbus_connection_call_cb (GObject      *object,
+                             GAsyncResult *result,
+                             gpointer      user_data)
+{
+  DexAsyncPair *async_pair = user_data;
+  GVariant *reply = NULL;
+  GError *error = NULL;
+
+  reply = g_dbus_connection_call_finish (G_DBUS_CONNECTION (object), result, &error);
+
+  if (error == NULL)
+    dex_async_pair_return_variant (async_pair, reply);
+  else
+    dex_async_pair_return_error (async_pair, error);
+
+  dex_unref (async_pair);
+}
+
+/**
+ * dex_dbus_connection_call:
+ * @bus_name:
+ * @object_path:
+ * @interface_name:
+ * @method_name:
+ * @parameters:
+ * @reply_type:
+ * @flags:
+ * @timeout_msec:
+ *
+ * Wrapper for g_dbus_connection_call().
+ *
+ * Returns: (transfer full): a #DexFuture that resolves to a #GVariant
+ *   or rejects with error.
+ *
+ * Since: 0.4
+ */
+DexFuture *
+dex_dbus_connection_call (GDBusConnection    *connection,
+                          const char         *bus_name,
+                          const char         *object_path,
+                          const char         *interface_name,
+                          const char         *method_name,
+                          GVariant           *parameters,
+                          const GVariantType *reply_type,
+                          GDBusCallFlags      flags,
+                          int                 timeout_msec)
+{
+  DexAsyncPair *async_pair;
+
+  g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), NULL);
+
+  async_pair = (DexAsyncPair *)dex_object_create_instance (DEX_TYPE_ASYNC_PAIR);
+
+  g_dbus_connection_call (connection,
+                          bus_name,
+                          object_path,
+                          interface_name,
+                          method_name,
+                          parameters,
+                          reply_type,
+                          flags,
+                          timeout_msec,
+                          async_pair->cancellable,
+                          dex_dbus_connection_call_cb,
+                          dex_ref (async_pair));
+
+  return DEX_FUTURE (async_pair);
+}
+
+#ifdef G_OS_UNIX
+static void
+dex_dbus_connection_call_with_unix_fd_list_cb (GObject      *object,
+                                               GAsyncResult *result,
+                                               gpointer      user_data)
+{
+  DexFutureSet *future_set = user_data;
+  DexAsyncPair *async_pair;
+  DexPromise *promise;
+  GUnixFDList *fd_list = NULL;
+  GVariant *reply = NULL;
+  GError *error = NULL;
+
+  g_assert (G_IS_DBUS_CONNECTION (object));
+  g_assert (DEX_IS_FUTURE_SET (future_set));
+
+  async_pair = DEX_ASYNC_PAIR (dex_future_set_get_future_at (future_set, 0));
+  promise = DEX_PROMISE (dex_future_set_get_future_at (future_set, 1));
+
+  g_assert (DEX_IS_ASYNC_PAIR (async_pair));
+  g_assert (DEX_IS_PROMISE (promise));
+
+  reply = g_dbus_connection_call_with_unix_fd_list_finish (G_DBUS_CONNECTION (object), &fd_list, result, &error);
+
+  g_assert (!fd_list || G_IS_UNIX_FD_LIST (fd_list));
+  g_assert (reply != NULL || error != NULL);
+
+  if (error == NULL)
+    {
+      dex_promise_resolve_object (promise, fd_list);
+      dex_async_pair_return_variant (async_pair, reply);
+    }
+  else
+    {
+      dex_promise_reject (promise, g_error_copy (error));
+      dex_async_pair_return_error (async_pair, error);
+    }
+
+  dex_unref (future_set);
+}
+
+/**
+ * dex_dbus_connection_call_with_unix_fd_list:
+ * @bus_name:
+ * @object_path:
+ * @interface_name:
+ * @method_name:
+ * @parameters:
+ * @reply_type:
+ * @flags:
+ * @timeout_msec:
+ * @fd_list: (nullable): a #GUnixFDList
+ *
+ * Wrapper for g_dbus_connection_call_with_unix_fd_list().
+ *
+ * Returns: (transfer full): a #DexFutureSet that resolves to a #GVariant.
+ *   The #DexFuture containing the resulting #GUnixFDList can be retrieved
+ *   with dex_future_set_get_future_at() with an index of 1.
+ *
+ * Since: 0.4
+ */
+DexFuture *
+dex_dbus_connection_call_with_unix_fd_list (GDBusConnection    *connection,
+                                            const char         *bus_name,
+                                            const char         *object_path,
+                                            const char         *interface_name,
+                                            const char         *method_name,
+                                            GVariant           *parameters,
+                                            const GVariantType *reply_type,
+                                            GDBusCallFlags      flags,
+                                            int                 timeout_msec,
+                                            GUnixFDList        *fd_list)
+{
+  DexAsyncPair *async_pair;
+  DexPromise *promise;
+  DexFuture *ret;
+
+  g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), NULL);
+  g_return_val_if_fail (!fd_list || G_IS_UNIX_FD_LIST (fd_list), NULL);
+
+  /* Will hold our GVariant result */
+  async_pair = (DexAsyncPair *)dex_object_create_instance (DEX_TYPE_ASYNC_PAIR);
+
+  /* Will hold our GUnixFDList result */
+  promise = dex_promise_new ();
+
+  /* Sent to user. Resolving will contain variant. */
+  ret = dex_future_all (DEX_FUTURE (async_pair), DEX_FUTURE (promise), NULL);
+
+  g_dbus_connection_call_with_unix_fd_list (connection,
+                                            bus_name,
+                                            object_path,
+                                            interface_name,
+                                            method_name,
+                                            parameters,
+                                            reply_type,
+                                            flags,
+                                            timeout_msec,
+                                            fd_list,
+                                            async_pair->cancellable,
+                                            dex_dbus_connection_call_with_unix_fd_list_cb,
+                                            dex_ref (ret));
+
+  return ret;
+}
+#endif
+
+static void
+dex_bus_get_cb (GObject      *object,
+                GAsyncResult *result,
+                gpointer      user_data)
+{
+  DexAsyncPair *async_pair = user_data;
+  GDBusConnection *bus;
+  GError *error = NULL;
+
+  bus = g_bus_get_finish (result, &error);
+
+  if (error == NULL)
+    dex_async_pair_return_object (async_pair, bus);
+  else
+    dex_async_pair_return_error (async_pair, error);
+
+  dex_unref (async_pair);
+}
+
+/**
+ * dex_bus_get:
+ * @bus_type:
+ *
+ * Wrapper for g_bus_get().
+ *
+ * Returns: (transfer full): a #DexFuture that resolves to a #GDBusConnection
+ *   or rejects with error.
+ *
+ * Since: 0.4
+ */
+DexFuture *
+dex_bus_get (GBusType bus_type)
+{
+  DexAsyncPair *async_pair;
+
+  async_pair = (DexAsyncPair *)dex_object_create_instance (DEX_TYPE_ASYNC_PAIR);
+
+  g_bus_get (bus_type,
+             async_pair->cancellable,
+             dex_bus_get_cb,
+             dex_ref (async_pair));
+
+  return DEX_FUTURE (async_pair);
+}
+
+static void
+dex_subprocess_wait_check_cb (GObject      *object,
+                              GAsyncResult *result,
+                              gpointer      user_data)
+{
+  DexAsyncPair *async_pair = user_data;
+  GError *error = NULL;
+
+  if (g_subprocess_wait_check_finish (G_SUBPROCESS (object), result, &error))
+    dex_async_pair_return_boolean (async_pair, TRUE);
+  else
+    dex_async_pair_return_error (async_pair, error);
+
+  dex_unref (async_pair);
+}
+
+/**
+ * dex_subprocess_wait_check:
+ * @subprocess: a #GSubprocess
+ *
+ * Creates a future that awaits for @subprocess to complete using
+ * g_subprocess_wait_check_async().
+ *
+ * Returns: (transfer full): a #DexFuture that will resolve when @subprocess
+ *   exits cleanly or reject upon signal or non-successful exit.
+ *
+ * Since: 0.4
+ */
+DexFuture *
+dex_subprocess_wait_check (GSubprocess *subprocess)
+{
+  DexAsyncPair *async_pair;
+
+  g_return_val_if_fail (G_IS_SUBPROCESS (subprocess), NULL);
+
+  async_pair = (DexAsyncPair *)dex_object_create_instance (DEX_TYPE_ASYNC_PAIR);
+
+  g_subprocess_wait_check_async (subprocess,
+                                 async_pair->cancellable,
+                                 dex_subprocess_wait_check_cb,
+                                 dex_ref (async_pair));
+
+  return DEX_FUTURE (async_pair);
+}
